//
// Copyright 2014 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.hibiscus.mahoe.santander.clearing;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.time.LocalDate;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.NullArgumentException;
import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Bean mit Ausgleichbuchungsinformationen
 *
 * @author Frank Mnster
 */
public class ClearingEntry {

    /**
     * Signum einer BigDecimal positiven Zahl
     */
    private static final int BIG_DECMINAL_POSITIVE = 1; // NOPMD

    /**
     * Der Ausgleichsbetrag
     */
    private final transient BigDecimal betrag;

    /**
     * Das F&auml;lligkitsdatum
     */
    private final transient Date datum;

    /**
     * Die Kreditkartennummer
     */
    private final transient String kartenNummer;

    /**
     * Der Karteninhaber
     */
    private final transient String inhaber;

    /**
     * Die IBAN
     */
    private final transient String iban;

    /**
     * Die BIC
     */
    private final transient String bic;

    /**
     * Das Abbuchungskonto
     */
    private transient Konto konto;

    /**
     * Erstellt eine neue Ausgleichsbuchung mit den angegebenen Daten.
     *
     * @param kartenNumber
     * @param inhaber
     * @param iban
     * @param bic
     * @param datum
     * @param betrag
     * @throws NullArgumentException
     */
    public ClearingEntry(final String kartenNumber, final String inhaber, final String iban, final String bic,
            final Date datum, final BigDecimal betrag) throws NullArgumentException {
        this.kartenNummer = kartenNumber;
        this.inhaber = inhaber;
        this.iban = iban;
        this.bic = bic;
        this.betrag = betrag;
        this.datum = datum;
        // Alle Felder gesetzt
        this.checkParameter();
        // Betrag skalieren
        this.betrag.setScale(2, RoundingMode.HALF_UP);
    }

    /**
     * &Uuml;berpr&uuml;pt die Parameter die zum Erstellen der Klasse angegeben wurden.
     *
     * @throws NullArgumentException
     */
    private void checkParameter() throws NullArgumentException {
        if (this.kartenNummer == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.cardNumberIsNUll")); //$NON-NLS-1$
        }
        if (this.inhaber == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.ownerIsNull")); //$NON-NLS-1$
        }
        if (this.iban == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.ibanIsNull")); //$NON-NLS-1$
        }
        if (this.bic == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.BicIsNull")); //$NON-NLS-1$
        }
        if (this.betrag == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.amountIsNull")); //$NON-NLS-1$
        }
    }

    /**
     * Erstellt die &Uuml;berweisung in Hibiscus.
     *
     * @return <i>true</i> falls die &Uuml;berweisung angelegt werden konnte, <i>false</i> sonst und wenn keine
     *         Expection ausgel&ouml;st wurde.
     * @throws NullArgumentException
     */
    public boolean saveEntry() throws NullArgumentException {
        if (this.konto == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.clearingAccountIsNull")); //$NON-NLS-1$
        }
        boolean saved = false;
        if (this.betrag.signum() == ClearingEntry.BIG_DECMINAL_POSITIVE) {
            try {
                final AuslandsUeberweisung clearingEntry = (AuslandsUeberweisung) Settings.getDBService()
                        .createObject(AuslandsUeberweisung.class, null);
                clearingEntry.setKonto(this.konto);
                // Umlaute ersetzten
                clearingEntry.setGegenkontoName(Utils.replaceUmlauts(this.inhaber));
                clearingEntry.setGegenkontoNummer(this.iban);
                clearingEntry.setGegenkontoBLZ(this.bic);
                final Double betrag = this.betrag.doubleValue();
                if (!betrag.isNaN()) {
                    clearingEntry.setBetrag(betrag);
                }
                clearingEntry.setZweck(this.kartenNummer);
                // Speichern
                clearingEntry.store();
                saved = true;
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("ClearingEntry.createSepaTransactionFailed") + e.getMessage(), e); //$NON-NLS-1$
            } catch (final ApplicationException e) {
                Logger.error(I18N.tr("ClearingEntry.saveSepaTransactionFailed") + e.getMessage(), e); //$NON-NLS-1$
            }
        }
        return saved;
    }

    /**
     * berprpt, ob eine Ausgleichsbuchung in den letzten 365 Tagen mit den selben Betrag schon existiert. Es werden
     * alle Hibiscus-Konten durchsucht.
     *
     * @return <i>true</i>, wenn die Ausgleichbuchung schon existiert, <i>false</i> sonst.
     */
    public boolean exists() {
        boolean exists = false;
        try {
            final HBCIDBService hcbiService = Settings.getDBService();
            // Erst mal die schon abgefragten Umstze
            exists = this.checkTransactions(hcbiService);
            if (!exists) {
                // Dann noch die SEPA-berweisungen
                exists = this.checkSEPA(hcbiService);
            }
        } catch (final RemoteException e) {
            // Auf true setzen, damit die berweisung nicht ggf. doppelt
            // angelegt wird.
            exists = true;
            final String msg = I18N.tr("ClearingEntry.searchClearingTransactionFailed"); //$NON-NLS-1$
            Logger.error(msg, e);
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        }
        return exists;
    }

    /**
     * &Uuml;berpr&uuml;ft, ob die Ausgleichsbuchung schon als angelegte Auslandsberweisung (SEPA) existiert.
     *
     * @param hcbiService
     *            der HBCIDBService
     * @return <i>true</i> falls die berweisung schon existiert, <i>false</i> sonst.
     * @throws RemoteException
     */
    @SuppressWarnings("PMD.NPathComplexity")
    private boolean checkSEPA(final HBCIDBService hcbiService) throws RemoteException {
        Boolean exists = false;
        final DBIterator<AuslandsUeberweisung> transactions = hcbiService.createList(AuslandsUeberweisung.class);
        while (transactions.hasNext()) {
            final AuslandsUeberweisung umsatz = transactions.next();
            // Nur die letzten 20 Tage for dem Flligsdatum berprfen
            if (umsatz.getAusfuehrungsdatum() != null) {
                final Date ausgefuehrtAm = umsatz.getAusfuehrungsdatum();
                final LocalDate umsatzDate = LocalDate.ofEpochDay(ausgefuehrtAm.getTime() / 86400000L);
                final LocalDate faellig = LocalDate.ofEpochDay(this.datum.getTime() / 86400000L).minusDays(20);
                if (umsatzDate.isBefore(faellig)) {
                    break;
                }
            }
            final String gkInhaber = StringUtils.isNotEmpty(umsatz.getGegenkontoName()) ? umsatz.getGegenkontoName()
                    : StringUtils.EMPTY;
            final double betrag = umsatz.getBetrag();
            final String kartenNummer = StringUtils.isNotEmpty(umsatz.getZweck()) ? umsatz.getZweck()
                    : StringUtils.EMPTY;
            final String myInhaber = Utils.replaceUmlauts(this.inhaber);
            if (myInhaber.toLowerCase(Locale.GERMAN).equals(gkInhaber.toLowerCase(Locale.GERMAN))
                    && this.kartenNummer.equals(kartenNummer) && this.betrag.doubleValue() == betrag) {
                Logger.debug("Equal..."); //$NON-NLS-1$
                exists = true;
                break;
            } else {
                Logger.debug("Not equal..."); //$NON-NLS-1$
            }
        }
        return exists;
    }

    /**
     * &Uuml;berpr&uuml;ft, ob die Ausgleichsbuchung schon als ausgefhrte berweisung existiert.
     *
     * @param hcbiService
     *            der HBCIDBService
     * @return <i>true</i>, falls die berweisung schon existiert, <i>false</i> sonst.
     * @throws RemoteException
     */
    private boolean checkTransactions(final HBCIDBService hcbiService) throws RemoteException {
        Boolean exists = false;
        final DBIterator<Konto> accounts = hcbiService.createList(Konto.class);
        while (accounts.hasNext() && !exists) {
            final Konto account = accounts.next();
            @SuppressWarnings("unchecked")
            final DBIterator<Umsatz> transactions = account.getUmsaetze(35);
            while (transactions.hasNext()) {
                final Umsatz umsatz = transactions.next();
                final String gkInhaber = StringUtils.isNotEmpty(umsatz.getGegenkontoName()) ? umsatz.getGegenkontoName()
                        : StringUtils.EMPTY;
                final double betrag = umsatz.getBetrag();
                final String kartenNummer = StringUtils.isNotEmpty(umsatz.getZweck()) ? umsatz.getZweck()
                        : StringUtils.EMPTY;
                // Compare
                if (this.inhaber.toLowerCase(Locale.GERMAN).equals(gkInhaber.toUpperCase(Locale.GERMAN))
                        && this.kartenNummer.equals(kartenNummer) && this.betrag.doubleValue() == betrag) {
                    Logger.debug("Equal..."); //$NON-NLS-1$
                    exists = true;
                    break;
                } else {
                    Logger.debug("Not equal..."); //$NON-NLS-1$
                }
            }
        }
        return exists;
    }

    /**
     * Setzt das Abbuchungskonto
     *
     * @param konto
     *            das Abbuchungskonto
     * @throws NullArgumentException
     * @throws RemoteException
     */
    public void setKonto(final Konto konto) throws NullArgumentException, RemoteException {
        if (konto == null) {
            throw new NullArgumentException(I18N.tr("ClearingEntry.clearingAccountIsNull")); //$NON-NLS-1$
        }
        final String ktoId = konto.getMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY);
        if (StringUtils.isNotEmpty(ktoId)) {
            this.konto = Utils.getKonto(ktoId);
        }
    }
}
