//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.web;

import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.messaging.ObjectChangedMessage;
import de.willuhn.jameica.hbci.messaging.SaldoMessage;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Minimale Kontoinformationen, die jedes Job auslesen muss.
 * 
 * @author Frank Mnster
 * 
 */
public abstract class AbstractWebAccountInformation implements // NOPMD
		WebAccountInformation {

	/**
	 * BIC
	 */
	private String bic;

	/**
	 * Der Kommentar
	 */
	private String comment;

	/**
	 * IBAN
	 */
	private String iban;

	/**
	 * Das letzte Umsatzdatum
	 */
	private Date lastTransactionDate; // NOPMD

	/**
	 * Das Saldo, wie auf der Webseite angezeigt
	 */
	private Double saldo;

	/**
	 * Verfgbarer Betrag
	 */
	private Double saldoAvailable;

	/**
	 * Das zugrundeliegende Konto
	 */
	protected Konto konto;

	/**
	 * Der Kontoauszugs-Speicherpfad
	 */
	private String savePath;

	/**
	 * Standardkonstruktor
	 */
	public AbstractWebAccountInformation() {
		// Absichtlich leer
	}

	/**
	 * @return the bic
	 */
	public String getBic() {
		return bic;
	}

	/**
	 * @return the comment
	 */
	@Override
	public String getComment() {
		return comment;
	}

	/**
	 * @return the iban
	 */
	public String getIban() {
		return iban;
	}

	/**
	 * @return the konto
	 */
	@Override
	public Konto getKonto() {
		return konto;
	}

	/**
	 * @return the lastTransactionDate
	 */
	@Override
	public Date getLastTransactionDate() {
		return lastTransactionDate;
	}

	/**
	 * Letztes Umsatzdatum schn formatiert
	 */
	@Override
	public String getLastTransactionDateAsString() {
		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
				Locale.GERMAN);
		return sdf.format(lastTransactionDate);
	}

	/**
	 * @return the saldo
	 */
	@Override
	public Double getSaldo() {
		return saldo;
	}

	/**
	 * @return the saldoAvailable
	 */
	@Override
	public Double getSaldoAvailable() {
		return saldoAvailable;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebAccountInformation#saveInformationInAccount(de.willuhn.util.ProgressMonitor)
	 */
	@Override
	public void saveInformationInAccount(final ProgressMonitor monitor) // NOPMD
			throws ApplicationException {

		if (konto == null) {
			throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.kontoIsNull")); //$NON-NLS-1$
		}

		// Saldo
		if (saldo != null) {
			monitor.setStatusText(I18N.tr("AbstractWebAccountInformation.updateSaldo")); //$NON-NLS-1$
			try {
				konto.setSaldo(saldo);
			} catch (RemoteException e) {
				Logger.error(I18N.tr("AbstractWebAccountInformation.setSaldoFailed"), e); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.setSaldoFailed"), e); //$NON-NLS-1$
			}
		}

		// Verfgbarer Betrag
		if (saldoAvailable != null) {
			monitor.setStatusText(I18N.tr("AbstractWebAccountInformation.updateSaldoAvailable")); //$NON-NLS-1$
			try {
				konto.setSaldoAvailable(saldoAvailable);
			} catch (RemoteException e) {
				Logger.error(I18N.tr("AbstractWebAccountInformation.setSaldoAvailableFailed"), e); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.setSaldoAvailableFailed"), e); //$NON-NLS-1$
			}
		}

		// Kommentar
		if (comment != null) {
			monitor.setStatusText(I18N.tr("AbstractWebAccountInformation.updateComment")); //$NON-NLS-1$
			try {
				konto.setKommentar(comment);

			} catch (RemoteException e) {
				Logger.error(I18N.tr("AbstractWebAccountInformation.updateCommentFailed"), e); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.updateCommentFailed"), e); //$NON-NLS-1$
			}
		}

		// IBAN
		if (iban != null) {
			monitor.setStatusText(I18N.tr("AbstractWebAccountInformation.updateIban")); //$NON-NLS-1$
			try {
				konto.setIban(iban.replaceAll("\\s", StringUtils.EMPTY)); //$NON-NLS-1$
			} catch (RemoteException e) {
				Logger.error(I18N.tr("AbstractWebAccountInformation.updateIbanFailed"), e); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.updateIbanFailed"), e); //$NON-NLS-1$
			}
		}

		// BIC
		if (bic != null) {
			monitor.setStatusText(I18N.tr("AbstractWebAccountInformation.updateBic")); //$NON-NLS-1$
			try {
				konto.setBic(bic.replaceAll("//s", StringUtils.EMPTY)); //$NON-NLS-1$
			} catch (RemoteException e) {
				Logger.error(I18N.tr("AbstractWebAccountInformation.updateBicFailed"), e); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.updateBicFailed"), e); //$NON-NLS-1$
			}
		}

		// Informationen von des Sub-Klassen speichern
		try {
			saveBankSpecificInfo(monitor);
		} catch (ApplicationException e) {
			Logger.error(I18N.tr("AbstractWebAccountInformation.setSpezificAccountInfoFailed"), e); //$NON-NLS-1$
			throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.setSpezificAccountInfoFailed"), //$NON-NLS-1$
					e);
		}

		try {
			konto.store();
			Application.getMessagingFactory().sendMessage(new SaldoMessage(konto));
			Application.getMessagingFactory().sendMessage(new ObjectChangedMessage(konto));
		} catch (RemoteException e) {
			Logger.error(I18N.tr("AbstractWebAccountInformation.storeKontoFailed"), e); //$NON-NLS-1$
			throw new ApplicationException(I18N.tr("AbstractWebAccountInformation.storeKontoFailed"), e); //$NON-NLS-1$
		}

	}

	/**
	 * @param bic
	 *            the bic to set
	 */
	public void setBic(final String bic) {
		this.bic = bic;
	}

	/**
	 * @param comment
	 *            the comment to set
	 */
	@Override
	public void setComment(final String comment) {
		this.comment = comment;
	}

	/**
	 * @param iban
	 *            the iban to set
	 */
	public void setIban(final String iban) {
		this.iban = iban;
	}

	/**
	 * @param konto
	 *            the konto to set
	 */
	@Override
	public void setKonto(final Konto konto) {
		this.konto = konto;
	}

	/**
	 * @param lastTransactionDate
	 *            the lastTransactionDate to set
	 */
	@Override
	public void setLastTransactionDate(final Date lastTransactionDate) { // NOPMD
		this.lastTransactionDate = lastTransactionDate;
	}

	/**
	 * @param saldo
	 *            the saldo to set
	 */
	@Override
	public void setSaldo(final Double saldo) {
		this.saldo = saldo;
	}

	/**
	 * @param saldoAvailable
	 *            the saldoAvailable to set
	 */
	@Override
	public void setSaldoAvailable(final Double saldoAvailable) {
		this.saldoAvailable = saldoAvailable;
	}

	/**
	 * Hier werden die webseitenspezifischen Kontoinformationen verarbeitet.
	 * Muss von der Child-Klasse implementiert werden.<br/>
	 * Ein speichern des Kontos ist nicht erforderlich. das macht die
	 * Super-Klasse.
	 * 
	 * @param monitor
	 * 
	 * @throws ApplicationException
	 */
	protected abstract void saveBankSpecificInfo(ProgressMonitor monitor) throws ApplicationException;

	/**
	 * @return the savePath
	 */
	public String getSavePath() {
		return savePath;
	}

	/**
	 * @param savePath
	 *            the savePath to set
	 */
	public void setSavePath(final String savePath) {
		this.savePath = savePath;
	}

}